/******************************************************************************
众安金融APP Native与H5 JS通信接口

1. 首先引入zaf.js文件
<script src="zaf.js"></script>

2. js中直接使用zaf这个全局对象调用native方法

// 调用native的方法，参数传递都是json对象或者没有参数
// 回调函数一般也是json对象，或者普通变量（如string，number）
zaf.getUserInfo({
  'success': function(userInfo) { // 返回值为json对象
	console.log(userInfo.username);
  },
  'fail': function() { // 无返回值
    console.log("fail");
  }
});

// 直接调用，无参数传递
zaf.close();

******************************************************************************/


(function() {
	
	// deep copy an object
	var cloneObj = function(obj) {
    var str, newobj = obj.constructor === Array ? [] : {};
    if(typeof obj !== 'object'){
        return;
    } else if(window.JSON){
        str = JSON.stringify(obj), //系列化对象
        newobj = JSON.parse(str); //还原
    } else {
        for(var i in obj){
            newobj[i] = typeof obj[i] === 'object' ? 
            cloneObj(obj[i]) : obj[i]; 
        }
    }
    return newobj;
	};

	// Bridge interface to SDK
	var bridge = window.bridge = {
		nativeSDKFiredReady: true,
		nativeCallInFlight: false,
		nativeCallQueue: [],
		callbacks: []
	};
	
	var CALLBACK_ID = 1;
	
	// Native 调用 JS, 主要为callback
	bridge.invokeJs = function(res) {
		console.log(res);
		var result = JSON.parse(res);
		var callbackId = result.callbackId;
		var func = this.callbacks[callbackId];
	
		var args = [];
	
		if (typeof result.response != 'undefined') {
			var response = result.response;
			args.push(response);
		}
		func.apply(this, args);
	};
  
	/** JS 调用 Native
	 * @param name The function name
	 * @param params The JSON Object
	 */
	bridge.invokeNative = function() {
		var args = Array.prototype.slice.call(arguments);
		var name = args.shift(); // the function name to invoke
		var params = args.shift(); // the params with JSON Object
		var newParams = {};
		
		if (typeof params != 'undefined') {
			newParams = cloneObj(params);
		
			if (typeof params.success === 'function') {
				var callbackId = CALLBACK_ID++;
				this.callbacks[callbackId] = params.success;
				newParams.success = callbackId;
			}
	
			if (typeof params.fail === 'function') {
				var callbackId = CALLBACK_ID++;
				this.callbacks[callbackId] = params.fail;
				newParams.fail = callbackId;
			}
		}

		var command = {
			'func': name,
			'params': newParams
		};
	
		var call = 'zaf://zaf.com/?command=' + encodeURIComponent(JSON.stringify(command));
		console.log(call);
		
		if (this.nativeCallInFlight) {
			this.nativeCallQueue.push(call);
		} else {
			this.nativeCallInFlight = true;
			window.location = call;
		}
		
	};
	
	bridge.nativeCallComplete = function(command) {
		if (this.nativeCallQueue.length === 0) {
			this.nativeCallInFlight = false;
			return;
		}

		var nextCall = this.nativeCallQueue.pop();
		window.location = nextCall;
	};
  
	// JS调用Native的方法
	var zaf = window.zaf = {};
	
	// 获取用户token
	zaf.getToken = function(params) {
		bridge.invokeNative('getToken', params);
	};
	
	// 获取用户信息
	zaf.getUserInfo = function(params) {
		bridge.invokeNative('getUserInfo', params);
	};
	
	// 扫一扫付款
	zaf.scanQRCode = function(params) {
		bridge.invokeNative('scanQRCode', params);
	};

	// 拍照
    zaf.takePhoto = function(params) {
    	bridge.invokeNative('takePhoto', params);
    };
	
	// 设置网页标题
	zaf.setTitle = function(params) {
		bridge.invokeNative('setTitle', params);
	};
	
	zaf.close = function() {
		bridge.invokeNative('close');
	};

	/*二期*/

	// 获取联系人
	zaf.getAddressBook = function(params) {
		bridge.invokeNative('getAddressBook', params);
	};

	// 扫描身份证
	zaf.scanIDCard = function(params) {
		bridge.invokeNative('scanIDCard', params);
	};

	// 设置右上角按钮
	zaf.setTopRightItem = function(params) {
		bridge.invokeNative('setTopRightItem', params);
	};

	// 人脸识别
	zaf.faceDetect = function(params) {
		bridge.invokeNative('faceDetect', params);
	};

	// 获取app信息
	zaf.getAppInfo = function(params) {
		bridge.invokeNative('getAppInfo', params);
	};

    // 获取app信息
    zaf.getIPAddress = function(params) {
        bridge.invokeNative('getIPAddress', params);
    };

    // 获取调用方设置的数据
    zaf.getBusinessInfo = function(params) {
       bridge.invokeNative('getBusinessInfo', params);
    };

    // 获取手机指纹
    zaf.getDeviceFingerprint = function(params) {
           bridge.invokeNative('getDeviceFingerprint', params);
    };

    // 获取银行卡
    zaf.scanBankID = function(params) {
           bridge.invokeNative('scanBankID', params);
    };

    zaf.notify = function(params) {
           bridge.invokeNative('notify', params);
    }

    zaf.showSettingDialog = function(params) {
               bridge.invokeNative('showSettingDialog', params);
    }

    zaf.hideNaviBack = function(params) {
                   bridge.invokeNative('hideNaviBack', params);
    }


}());